<?php
/* vim: set expandtab tabstop=4 softtabstop=4 shiftwidth=4: */
//
// +----------------------------------------------------------------------+
// | PHP version 4.0                                                      |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2001 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Piotr Klaban <makler@man.torun.pl>                          |
// +----------------------------------------------------------------------+
//
// $Id: lang.pl.php,v 1.3 2002/11/26 10:33:35 makler Exp $
//
// Numbers_Words class extension to spell numbers in Polish.
//

require_once("PEAR.php");
require_once("Numbers/Words.php");

/**
* Class for translating numbers into Polish.
*
* @author Piotr Klaban
*/
class Numbers_Words_pl extends Numbers_Words
{

    // {{{ properties
    
    /**
    * Locale name
    * @var string
    */
    var $locale      = 'pl';

    /**
    * Language name in English
    * @var string
    */
    var $lang        = 'Polish';

    /**
    * Native language name
    * @var string
    */
    var $lang_native = 'polski';

    /**
    * The word for the minus sign
    * @var string
    */
    var $_minus = 'minus'; // minus sign
    
    /**
    * The sufixes for exponents (singular and plural)
    * Names based on:
    * mathematical tables, my memory, and also:
    * http://ux1.math.us.edu.pl/~szyjewski/FAQ/liczby/iony.htm
    * @var array
    */
    var $_exponent = array(
       // potga dziesitki => liczba pojedyncza, podwjna, mnoga
        0 => array('','',''),
        3 => array('tysic','tysice','tysicy'),
        6 => array('milion','miliony','milionw'),
        9 => array('miliard','miliardy','miliardw'),
       12 => array('bilion','biliony','bilionw'),
       15 => array('biliard','biliardy','biliardw'),
       18 => array('trylion','tryliony','trylionw'),
       21 => array('tryliard','tryliardy','tryliardw'),
       24 => array('kwadrylion','kwadryliony','kwadrylionw'),
       27 => array('kwadryliard','kwadryliardy','kwadryliardw'),
       30 => array('kwintylion','kwintyliony','kwintylionw'),
       33 => array('kwintyliiard','kwintyliardy','kwintyliardw'),
       36 => array('sekstylion','sekstyliony','sekstylionw'),
       39 => array('sekstyliard','sekstyliardy','sekstyliardw'),
       42 => array('septylion','septyliony','septylionw'),
       45 => array('septyliard','septyliardy','septyliardw'),
       48 => array('oktylion','oktyliony','oktylionw'),
       51 => array('oktyliard','oktyliardy','oktyliardw'),
       54 => array('nonylion','nonyliony','nonylionw'),
       57 => array('nonyliard','nonyliardy','nonyliardw'),
       60 => array('decylion','decyliony','decylionw'),
       63 => array('decyliard','decyliardy','decyliardw'),
      100 => array('centylion','centyliony','centylionw'),
      103 => array('centyliard','centyliardy','centyliardw'),
      120 => array('wicylion','wicylion','wicylion'),
      123 => array('wicyliard','wicyliardy','wicyliardw'),
      180 => array('trycylion','trycylion','trycylion'),
      183 => array('trycyliard','trycyliardy','trycyliardw'),
      240 => array('kwadragilion','kwadragilion','kwadragilion'),
      243 => array('kwadragiliard','kwadragiliardy','kwadragiliardw'),
      300 => array('kwinkwagilion','kwinkwagilion','kwinkwagilion'),
      303 => array('kwinkwagiliard','kwinkwagiliardy','kwinkwagiliardw'),
      360 => array('seskwilion','seskwilion','seskwilion'),
      363 => array('seskwiliard','seskwiliardy','seskwiliardw'),
      420 => array('septagilion','septagilion','septagilion'),
      423 => array('septagiliard','septagiliardy','septagiliardw'),
      480 => array('oktogilion','oktogilion','oktogilion'),
      483 => array('oktogiliard','oktogiliardy','oktogiliardw'),
      540 => array('nonagilion','nonagilion','nonagilion'),
      543 => array('nonagiliard','nonagiliardy','nonagiliardw'),
      600 => array('centylion','centyliony','centylionw'),
      603 => array('centyliard','centyliardy','centyliardw'),
  6000018 => array('milinilitrylion','milinilitryliony','milinilitrylionw')
        );

    /**
    * The array containing the digits (indexed by the digits themselves).
    * @var array
    */
    var $_digits = array(
        0 => 'zero', 'jeden', 'dwa', 'trzy', 'cztery',
        'pi', 'sze', 'siedem', 'osiem', 'dziewi'
    );

    /**
    * The word separator
    * @var string
    */
    var $_sep = ' ';

    // }}}
    // {{{ toWords()

    /**
     * Converts a number to its word representation
     * in Polish language
     *
     * @param  integer $num   An integer between -infinity and infinity inclusive :)
     *                        that need to be converted to words
     * @param  integer $power The power of ten for the rest of the number to the right.
     *                        Optional, defaults to 0.
     * @param  integer $powsuffix The power name to be added to the end of the return string.
     *                        Used internally. Optional, defaults to ''.
     *
     * @return string  The corresponding word representation
     *
     * @access public
     * @author Piotr Klaban <makler@man.torun.pl>
     * @since  PHP 4.2.3
     */
    function toWords($num, $power = 0, $powsuffix = '') {
      $ret = '';        

      // add a minus sign
      if (substr($num, 0, 1) == '-') {
        $ret = $this->_sep . $this->_minus;
        $num = substr($num, 1);
      }
        
      // strip excessive zero signs and spaces
      $num = trim($num);
      $num = preg_replace('/^0+/','',$num);
        
      if (strlen($num) > 3) {
          $maxp = strlen($num)-1;
          $curp = $maxp;
          for ($p = $maxp; $p > 0; --$p) { // power
            
            // check for highest power
            if (isset($this->_exponent[$p])) {
              // send substr from $curp to $p
              $snum = substr($num, $maxp - $curp, $curp - $p + 1);
              $snum = preg_replace('/^0+/','',$snum);
              if ($snum !== '') {
                  $cursuffix = $this->_exponent[$power][count($this->_exponent[$power])-1];
                  if ($powsuffix != '')
                    $cursuffix .= $this->_sep . $powsuffix;
                  $ret .= $this->toWords($snum, $p, $cursuffix);
              }
              $curp = $p - 1;
              continue;
            }
          }
          $num = substr($num, $maxp - $curp, $curp - $p + 1);
          if ($num == 0) {
              return $ret;
          }
      } elseif ($num == 0 || $num == '') {
        return $this->_sep . $this->_digits[0];
      }
    
      $h = $t = $d = 0;
      
      switch(strlen($num)) {
        case 3:
          $h = (int)substr($num,-3,1);

        case 2:
          $t = (int)substr($num,-2,1);

        case 1:
          $d = (int)substr($num,-1,1);
          break;

        case 0:
          return;
          break;
      }

      switch ($h) {
      case 9:
          $ret .= $this->_sep . 'dziewiset';
          break;

      case 8:
          $ret .= $this->_sep . 'osiemset';
          break;

      case 7:
          $ret .= $this->_sep . 'siedemset';
          break;

      case 6:
          $ret .= $this->_sep . 'szeset';
          break;

      case 5:
          $ret .= $this->_sep . 'piset';
          break;

      case 4:
          $ret .= $this->_sep . 'czterysta';
          break;

      case 3:
          $ret .= $this->_sep . 'trzysta';
          break;

      case 2:
          $ret .= $this->_sep . 'dwiecie';
          break;

      case 1:
          $ret .= $this->_sep . 'sto';
          break;
      }

      switch ($t) {
      case 9:
      case 8:
      case 7:
      case 6:
      case 5:
          $ret .= $this->_sep . $this->_digits[$t] . 'dziesit';
          break;

      case 4:
          $ret .= $this->_sep . 'czterdzieci';
          break;

      case 3:
          $ret .= $this->_sep . 'trzydzieci';
          break;

      case 2:
          $ret .= $this->_sep . 'dwadziecia';
          break;

      case 1:
          switch ($d) {
          case 0:
              $ret .= $this->_sep . 'dziesi';
              break;

          case 1:
              $ret .= $this->_sep . 'jedenacie';
              break;

          case 2:
          case 3:
          case 7:
          case 8:
              $ret .= $this->_sep . $this->_digits[$d] . 'nacie';
              break;

          case 4:
              $ret .= $this->_sep . 'czternacie';
              break;

          case 5:
              $ret .= $this->_sep . 'pitnacie';
              break;

          case 6:
              $ret .= $this->_sep . 'szesnacie';
              break;

          case 9:
              $ret .= $this->_sep . 'dziewitnacie';
              break;
          }
          break; 
      }

      if ($t != 1 && $d > 0)
        $ret .= $this->_sep . $this->_digits[$d];
  
      if ($t == 1)
        $d = 0;

      if (( $h + $t ) > 0 && $d == 1)
        $d = 0;

      if ($power > 0) {
        if (isset($this->_exponent[$power]))
          $lev = $this->_exponent[$power];

        if (!isset($lev) || !is_array($lev))
          return null;
 
        switch ($d) {
          case 1:
            $suf = $lev[0];
            break;
          case 2:
          case 3:
          case 4:
            $suf = $lev[1];
            break;
          case 0:
          case 5:
          case 6:
          case 7:
          case 8:
          case 9:
            $suf = $lev[2];
            break;
        }
        $ret .= $this->_sep . $suf;
      }
  
      if ($powsuffix != '')
        $ret .= $this->_sep . $powsuffix;

      return $ret;
    }
    // }}}
}

?>
